export declare type IdEntity = {
    id: string;
};
export declare type NumericIdEntity = {
    id: number;
};
/** The core value converter */
export interface IValueConverter<TFrom, TTo> {
    convert?: IOneWayValueConverter<TFrom, TTo>;
    convertBack?: IOneWayValueConverter<TTo, TFrom>;
}
/** The core one way value converter */
export interface IOneWayValueConverter<TFrom, TTo> {
    (from: TFrom): TTo;
}
/** The core 'text' input value converter (always requires a string ) */
export interface IInputValueConverter<TDataPropValue> extends IValueConverter<TDataPropValue, string> {
}
/** The core 'checkbox' input value converter (always requires a boolean) */
export declare class CheckboxValueConverter<TDataPropValue> implements IValueConverter<TDataPropValue, boolean> {
    private trueValue;
    private falseValue;
    constructor(trueValue: TDataPropValue, falseValue: TDataPropValue);
    convert(data: TDataPropValue): boolean;
    convertBack(value: boolean): TDataPropValue;
}
/** Converts numeric id of property item to string  */
export declare class IdEntityValueConverter implements IInputValueConverter<IdEntity> {
    convertBack: IOneWayValueConverter<string, IdEntity>;
    constructor(convertBack: IOneWayValueConverter<string, IdEntity>);
    convert(data: IdEntity): string;
}
/** Pass-through converter - used for setting string id value on bound property item */
export declare class IdValueConverter implements IInputValueConverter<string> {
    convertBack: IOneWayValueConverter<string, string>;
    constructor(convertBack: IOneWayValueConverter<string, string>);
    convert(data: string): string;
}
/** Converts numeric id of item to string  */
export declare class NumericIdEntityValueConverter implements IInputValueConverter<NumericIdEntity> {
    convertBack: IOneWayValueConverter<string, NumericIdEntity>;
    constructor(convertBack: IOneWayValueConverter<string, NumericIdEntity>);
    convert(data: NumericIdEntity): string;
}
/** Converts string to number - used for setting numeric id value on bound element */
export declare class NumericIdValueConverter implements IInputValueConverter<number> {
    convertBack: IOneWayValueConverter<string, number>;
    constructor(convertBack: IOneWayValueConverter<string, number>);
    convert(data: number): string;
}
/** The Default pass-through Value converter */
export declare class DefaultValueConverter implements IInputValueConverter<string> {
    convert(data: string): string;
    convertBack(value: string): string;
    static instance: DefaultValueConverter;
}
export interface INumericOptions {
    /** allow negative values? */
    allowNegative?: boolean;
    /** number of decimals */
    decimals?: number;
    /** min value */
    min?: number;
    /** max value */
    max?: number;
    /** range step */
    step?: number;
}
/** A Numeric Value converter written to handle Text Input */
export declare class NumericValueConverter implements IInputValueConverter<number> {
    private options;
    constructor(options?: INumericOptions);
    convert(data: number): string;
    convertBack(value: string): number;
    static instance: NumericValueConverter;
}
